/*global CloseableView*/
/*global Toolkit, _*/
(function () {
	/**
	 * @param model {Toolkit.Models.App}
	 */
	Toolkit.Views.BrowseModulesView = CloseableView.extend({
		breadcrumbs: function () {
			if (this.options.breadcrumbs) {
				return this.appBreadcrumbs(this.model, _.map(this.options.breadcrumbs, function (breadcrumb) { breadcrumb['target'] = 'moduleindex'; return breadcrumb; }));
			} else {
				return this.appBreadcrumbs(this.model, {
					text: this.options.breadcrumbs || "...",
					current: true
				});
			}
		},


		initialize: function () {
			this.listenTo(this.model, 'change:name', this.updateBreadcrumbs);
			this.listenToWhenAvailable(this.model, 'project', 'change:name', this.updateBreadcrumbs);
			
			var me = this;
			this.options.listener = function (evt) {
				var local;
				if (Toolkit.server.indexOf(evt.origin) === 0) {
					if (evt.data.method == 'setBreadcrumbs') {
						me.options.breadcrumbs = evt.data.breadcrumbs;
						me.updateBreadcrumbs();
					} else if (evt.data.method == 'getModuleDetailsForApp') {
						this.doWhenAvailable(me.model, 'local', function (local) {
							this.doWhenAvailable(local, 'config', function () {
								var modules = local.get('config')['modules'];
								if (!(evt.data.module in modules)) {
									evt.source.postMessage({
										method: 'getModuleDetailsForApp',
										enabled: false,
										uuid: me.model.get("uuid"),
										project: me.model.get("project_id")
									}, evt.origin);
								} else {
									evt.source.postMessage({
										method: 'getModuleDetailsForApp',
										enabled: true,
										version: modules[evt.data.module]['version'],
										uuid: me.model.get("uuid"),
										project: me.model.get("project_id")
									}, evt.origin);
								}
							});
						});
					} else if (evt.data.method == 'setModuleVersionForApp') {
						var app = me.model;
						local = me.model.get("local");
						var setModuleVersionForApp = function () {
							local.updateModule(evt.data.module, evt.data.version);
							
							evt.source.postMessage({
								method: 'getModuleDetailsForApp',
								enabled: true,
								version: evt.data.version,
								uuid: app.get("uuid"),
								project: app.get("project_id")
							}, evt.origin);
						};

						if (evt.data.trusted || Toolkit.authStatus.get("thirdparty_module_warning")) {
							setModuleVersionForApp();
						} else {
							// display a warning first time the user adds an untrusted module
							var warning = new Toolkit.Views.ModuleTrustWarning({
								onAccept: function () {
									setModuleVersionForApp();
								}
							});
							warning.show();
						}
					}
				}
			};
			window.addEventListener("message", this.options.listener, false);
		},

		close: function () {
			window.removeEventListener("message", this.options.listener, false);
			Toolkit.Views.BrowseModulesView.__super__.close.call(this);
		},

		render: function () {
			this.updateBreadcrumbs();
			if (this.options.name && this.options.version) {
				this.$el.append(
					"<iframe id='moduleindex' scrolling='no' style='width: 100%' src='"+Toolkit.websiteUrl()+"embed/modules/" + this.options.name + "/" + this.options.version + "/'></iframe>"
				);
			} else {
				this.$el.append(
					"<iframe id='moduleindex' scrolling='no' style='width: 100%' src='"+Toolkit.websiteUrl()+"embed/modules/_/featured'></iframe>"
				);
			}
			return this;
		}
	});
}());
