/*global Toolkit*/
/*global _*/
/*global $*/
/*global Mustache*/
/*global fixBootstrapButtonEvent*/
(function () {
	var PublicModuleView = Toolkit.Views.PublicModuleView = Toolkit.Views.GenericModuleConfigView.extend({
		template: 'apps/publicModules/module',

		events: {
			"click .update-version": "updateVersion"
		},

		initialize: function () {
			Toolkit.Views.GenericModuleConfigView.prototype.initialize.call(this);
			this.listenToWhenAvailable(this.model, 'object', 'change:status', this.renderIfNecessary);
		},

		templateAttributes: function () {
			return _.extend(Toolkit.Views.GenericModuleConfigView.prototype.templateAttributes.call(this), {
				'status': this.model.get("object").getStatusesWithTag('module-'+this.model.get("moduleVersion").get("module__name"))
			});
		},

		updateVersion: function (e) {
			e.preventDefault();
			this.model.get("local").updateModule(this.model.get("moduleVersion").get("module__name"), $(e.target).attr('data-version'));
		}
	});

	_.extend(PublicModuleView.prototype.events, PublicModuleView.__super__.events);

	Toolkit.Views.ModulesView = Toolkit.Views.GenericAppView.extend({
		active: ['Public', 'Modules'],

		ContentView: Toolkit.Views.GenericModulesConfigView.extend({
			template: 'apps/publicModules/config',

			ModuleView: PublicModuleView,

			moduleType: 'public',

			events: {
				"click .module-add": "clickModuleAdd",
				"focus .search-query": "onFocusSearch",
				"blur .search-query": "onBlurSearch",
				"keyup .search-query": "onKeyupSearch"
			},

			clickModuleAdd: function (e) {
				this.$(".search-query").val("");
				fixBootstrapButtonEvent(e);
				var name = this.$(e.target).attr('data-name');
				var version = this.$(e.target).attr('data-version');
				var trusted = this.$(e.target).attr('data-trusted') === "true";

				var local = this.model.get('app').get("local");
				if (trusted || Toolkit.authStatus.get("thirdparty_module_warning")) {
					local.addModule(name, version);
				} else {
					// display a warning first time the user adds an untrusted module
					var warning = new Toolkit.Views.ModuleTrustWarning({
						onAccept: function () {
							local.addModule(name, version);
						}
					});
					warning.show();
				}
			},

			onFocusSearch: function () {
				var browseUrl = '#apps/'+this.model.get('app').get('uuid')+'/browseModules',
					query = this.$(".search-query").val(),
					title;

				if (query === "") {
					title = "<span class='search-results-header'>Featured modules</span> ";
				} else {
					title = "<span class='search-results-header'>Search results</span> <i class='spinner icon-cog icon-spin'></i> ";
				}
				title += "<a href='"+browseUrl+"' class='pull-right'>show module index</a>";
				this.$(".search-query").popover({
					title: title,
					content: "<div class='row-fluid search-results'></div>",
					placement: "bottom",
					trigger: "manual",
					html: true
				}).popover("show");
				this.moduleSearch(query);
			},

			onBlurSearch: function () {
				var me = this;
				setTimeout(function () {
					me.$(".search-query").popover("destroy");
				}, 100); // give any click events time to propogate :-P
			},

			onKeyupSearch: function (e) {
				this.moduleSearch(e.target.value);
			},

			moduleSearch: _.throttle(function (query) {
				var me = this;
			
				var url = "modules/public/search/?" + $.param({ q: query, project: me.model.get("project").id });
				this.$('.spinner').addClass('spinner-on');

				var rpc = Toolkit.rpcSession.remoteCall("api.get", {
					method: url,
					cookies: Toolkit.cookies
				});
				rpc.on("success", function (result) {
					console.log("SEARCHED", result.data.data.modules[0]);
					me.$('.spinner').removeClass('spinner-on');
					var search_results = _.map(result.data.data.modules, function (module) {
						return _.extend(module, {
							url: "#apps/" + me.model.get("app").get("uuid") + "/browseModuleDetails/" + module.name + "/current",
							added: !!me.model.get("local").getConfig()['modules'][module.name]
						});
					});
					me.$(".search-results").html(Mustache.to_html(Toolkit.Templates['apps/publicModules/search_results'], {
						modules: search_results
					}));
					me.$(".search-results-header").text(
						query === "" ? "Featured modules" : "Search results"
					);
				});
			}, 500)
		})
	});

	Toolkit.Views.InspectorModulesView = Toolkit.Views.GenericModuleView.extend({
		active: 'Public',

		ContentView: Toolkit.Views.GenericModulesConfigView.extend({
			template: 'apps/publicModules/config',

			ModuleView: PublicModuleView,

			moduleType: 'public',

			events: {
				"click .module-add": "clickModuleAdd",
				"focus .search-query": "onFocusSearch",
				"blur .search-query": "onBlurSearch",
				"keyup .search-query": "onKeyupSearch"
			},

			templateAttributes: function () {
				var attr = Toolkit.Views.GenericModulesConfigView.prototype.templateAttributes.apply(this, arguments),
					local = this.model.get('module').get('local');

				var disabled = !local ||
					!local.getPlatformVersion() ||
					!Toolkit.platforms.get("platforms").get(local.getPlatformVersion()) ||
					!_.contains(Toolkit.platforms.get("platforms").get(local.getPlatformVersion()).get("features"), "modules-inspector-config");

				return _.extend(attr, {
					disabled: disabled
				});
			},

			clickModuleAdd: function (e) {
				this.$(".search-query").val("");
				fixBootstrapButtonEvent(e);
				var name = this.$(e.target).attr('data-name');
				var version = this.$(e.target).attr('data-version');
				var trusted = this.$(e.target).attr('data-trusted') === "true";

				var local = this.model.get('module').get("local");
				if (trusted || Toolkit.authStatus.get("thirdparty_module_warning")) {
					local.addModule(name, version);
				} else {
					// display a warning first time the user adds an untrusted module
					var warning = new Toolkit.Views.ModuleTrustWarning({
						onAccept: function () {
							local.addModule(name, version);
						}
					});
					warning.show();
				}
			},

			onFocusSearch: function () {
				this.$(".search-query").popover({
					title: "Search results <i class='spinner icon-cog icon-spin'></i>",
					content: "<div class='row-fluid search-results'></div>",
					placement: "bottom",
					trigger: "manual",
					html: true
				}).popover("show");
				this.moduleSearch(this.$(".search-query").val());
			},

			onBlurSearch: function () {
				var me = this;
				setTimeout(function () {
					me.$(".search-query").popover("hide");
				}, 100); // give any click events time to propogate :-P
			},

			onKeyupSearch: function (e) {
				this.moduleSearch(e.target.value);
			},

			moduleSearch: _.throttle(function (query) {
				var me = this;
			
				if (query === "") {
					me.$(".search-results").html("");
					return;
				}
				var url = "modules/public/search/?" + $.param({ q: query, project: me.model.get("project").id });
				this.$('.spinner').addClass('spinner-on');

				var rpc = Toolkit.rpcSession.remoteCall("api.get", {
					method: url,
					cookies: Toolkit.cookies
				});
				rpc.on("success", function (result) {
					console.log("SEARCHED", result.data.data.modules[0]);
					me.$('.spinner').removeClass('spinner-on');
					var search_results = _.map(result.data.data.modules, function (module) {
						return _.extend(module, {
							added: !!me.model.get("module").get("local").getConfig()['modules'][module.name]
						});
					});
					me.$(".search-results").html(Mustache.to_html(Toolkit.Templates['apps/publicModules/search_results'], {
						modules: search_results
					}));
				});
			}, 500)
		})
	});
}());
