/*global Backbone*/
/*global Toolkit*/
/*global $*/
/*global _*/
// To use set data-dirpicker="id" or data-filepicker="id"
// where id is the id of the element you want to be filled in with the selected path
// i.e.
// <input type="text" id="path" />
// <a href="#" data-dirpicker="path">Choose directory</a>
//
// set data-pickerstart="/path/to/somewhere" to where
// you want the filepicker to initially open
//
// set data-pickerroot="/path/to/somewhere" to where you want files picked to be relative to.
// e.g.
// <a href="#" data-filepicker="path" data-pickerroot="/path/to/somewhere">
// if user picks "/path/to/somewhere/else/myfile.png"
// then the picker will populate #path with "else/myfile.png"

(function () {
	var DirectoryModel = Backbone.Model.extend({
		update: function (directory, root, create) {
			var state = this;
			var getList = function () {
				var rpc = Toolkit.rpcSession.remoteCall('directory.list_contents', {
					path: directory || '',
					root: state.get('root') || '',
					show_hidden: state.get('show_hidden') || false
				});

				rpc.on('success', function (e) {
					state.set('path_string', directory);
					state.set(e.data);
				});
			};

			if (create) {
				var rpc = Toolkit.rpcSession.remoteCall('directory.create', {
					path: directory
				});

				rpc.on('success', function () {
					getList();
				});
			} else {
				getList();
			}
		},

		refresh: function () {
			this.update(this.get('path_string'));
		}
	});
	
	var PickerView = Backbone.View.extend({
		tagName: 'div',
		className: 'filepicker',
		events: {
			'click .pathlink': 'pathclick',
			'click .filelink': 'fileclick',
			'click .createdir': 'createclick',
			'click #filepickerselect': 'select',
			'click .outsidemodal': 'close',
			'change .showhidden': 'showHiddenChange'
		},
		close: function () {
			this.trigger("cancel");
			return false;
		},
		showHiddenChange: function (e) {
			this.model.set('show_hidden', !!$(e.currentTarget).attr('checked'));
			this.model.refresh();
		},
		select: function () {
			var pathParts = this.model.get('path');
			// always store / in the config
			var sep = '/';

			if (this.model.get('root')) {
				// Our filepicker is relative to a chosen folder, remove everything up to that root
				var rootParts = this.model.get('root_parts');
				pathParts = pathParts.slice(rootParts.length);
			}

			var path = pathParts.join(sep);

			if (path.length !== 0) {
				path += sep;
			}

			if (pathParts[0] == sep) {
				path = path.substring(1);
			}

			if (this.model.get('selected')) {
				path += this.model.get('selected');
			}

			this.model.get('callback') && this.model.get('callback')(path);
			this.close();
			return false;
		},
		pathclick: function (e) {
			this.model.set('selected', null);
			this.model.update($(e.target).attr('data-path'));
			return false;
		},
		fileclick: function (e) {
			this.model.set('selected', $(e.target).attr('data-file'));
			return false;
		},
		createclick: function () {
			var name = prompt("Enter a name for the new directory");
			if (name) {
				this.model.set('selected', null);
				this.model.update(this.model.get('path').join(this.model.get('separator'))+this.model.get('separator')+name, true);
			}
			return false;
		},
		initialize: function () {
			this.model.on('change', this.render, this);
		},
		render: function () {
			$('body').css('min-height', '600px');
			
			$(this.el).html(Toolkit.Templates['component/filepicker']);
			var $el = $(this.el);
			// Render current path
			var sep = this.model.get('separator');
			var first = true;
			var curPath = "";
			var root = this.model.get('root');
			var rootParts = this.model.get('root_parts');

			if (this.model.get('show_hidden')) {
				this.$('.showhidden').attr('checked', 'checked');
			}

			this.model.get('path') && _.each(this.model.get('path'), function (curDir, i) {
				curPath += curDir;
				if (curPath != sep) {
					curPath += sep;
				}

				// If a root folder for the picker was specified,
				// don't display any folders before the root.
				if (root && i < rootParts.length - 1) {
					return;
				}

				if (!first) {
					$('.path', $el).append('<span> '+sep+' </span>');
				}

				$('.path', $el).append('<a href="#" class="pathlink" data-path="'+curPath+'">'+curDir+'</a>');
				first = false;
			});

			if (this.model.get('selected')) {
				$('.path', $el).append('<span> '+sep+' </span>');
				$('.path', $el).append('<span>'+this.model.get('selected')+'</a>');
			}

			$('.contents', $el).append('<div class="create"><a href="#" class="createdir"><i class="icon-plus-sign"></i> Create directory</a></div>');
			// Render files in current directory
			var count = 0;
			var model = this.model;
			if (this.model.get('contents') && this.model.get('contents').length) {
				_.each(this.model.get('contents'), function (file) {
					if (file[1]) {
						count++;
						$('.contents', $el).append('<div class="dir"><a href="#" class="pathlink" data-path="'+curPath+file[0]+'"><i class="icon-folder-open"></i> '+file[0]+'</a></div>');
					} else if (model.get('type') == 'file') {
						count++;
						$('.contents', $el).append('<div class="file"><a href="#" class="filelink" data-file="'+file[0]+'"><i class="icon-file"></i> '+file[0]+'</a></div>');
					}
				});
			}
			if (count === 0) {
				$('.contents', $el).append('<div>Empty directory</div>');
			}
			return this;
		}
	});
	
	var filepicker = function (options) {
		var state = new DirectoryModel({
			type: options.type,
			root: options.root,
			callback: options.callback
		});

		state.update(options.start);
		var pickerView = new PickerView({
			model: state
		});

		var modalView = new Toolkit.Views.ModalDialog({
			title: 'Choose  '+ options.type,
			contentView: pickerView
		});
		modalView.show();
	};
	
	$('[data-dirpicker]').live('click', function () {
		var id = $(this).attr('data-dirpicker');
		var start = $(this).attr('data-pickerstart') || '';
		var root = $(this).attr('data-pickerroot') || '';
		filepicker({
			type: 'directory',
			start: start,
			root: root,
			callback: function (path) {
				$(document.getElementById(id)).val(path).trigger('change');
			}
		});
		return false;
	});

	$('[data-filepicker]').live('click', function () {
		var id = $(this).attr('data-filepicker');
		var start = $(this).attr('data-pickerstart') || '';
		var root = $(this).attr('data-pickerroot') || '';
		filepicker({
			type: 'file',
			start: start,
			root: root,
			callback: function (path) {
				$(document.getElementById(id)).val(path).trigger('change');
			}
		});
		return false;
	});
})();
