/*global Backbone*/
/*global Toolkit*/
/*global _*/
/*global $*/
(function () {
	var KEY_ENTER = 13;

	// to use, extend and override 'template' and 'submit'
	// see e.g. AppCreateView
	var FormView = Toolkit.Views.FormView = Toolkit.Views.ViewWithTemplate.extend({
		events: {
			"click .submit": "_clickedSubmit",
			"keydown input": "_keydownInInput"
		},

		_keydownInInput: function (e) {
			if (e.which === KEY_ENTER) {
				e.preventDefault();
				this._trySubmit();
			}
		},

		_clickedSubmit: function (e) {
			e.preventDefault();
			this._trySubmit();
		},

		_trySubmit: function () {
			this.tempModel.set('errors', {});
			if (this._spinner.get('status') !== 'on') {
				var form = {};

				var $inputs = this.$('input,select,textarea');
				$inputs.each(function () {
					if ($(this).attr('type') === "checkbox") {
						form[this.name] = $(this).is(":checked");
					} else {
						form[this.name] = $(this).val();
					}
				});

				if (!this._html5Validate()) {
					return;
				}

				this._submitStart();
				this.submit(form);
			}
		},

		_html5Validate: function () {
			var $form = this.$('form');
			if ($form.length > 0 && !$form.get(0).checkValidity()) {
				// HTML5 validation fails: submit form to force browser to show UI
				$('input[type=submit]', $form).click();
				return false;
			}
			return true;
		},

		_submitStart: function () {
			this.$('.submit').addClass('disabled');
			this.$('input').attr('readonly', true);
			this.$('.spinner').addClass('spinner-on');
		},

		_submitFinish: function () {
			this.$('.submit').removeClass('disabled');
			this.$('input').attr('readonly', false);
			this.$('.spinner').removeClass('spinner-on');
		},

		_handleFormErrors: function () {
			var form = this;
			return function (e) {
				if (e.extra.errors) {
					form.tempModel.set('errors', e.extra.errors);
				} else {
					this.defaults.error(e);
				}
				form._submitFinish();
			};
		},

		initialize: function () {
			FormView.__super__.initialize.apply(this, arguments);
			this._spinner = new Backbone.Model({status: 'off'});
			this.tempModel = new Backbone.Model();
			this.tempModel.bind('change:errors', this._renderErrors, this);
		},

		_renderGeneralErrors: function (generalErrors) {
			var $generalErrors = this.$('.general-errors');
			$generalErrors.html('').show(100);
			_(generalErrors).each(function (errorText) {
				$generalErrors.append('<div>'+errorText+'</div>');
			});
		},

		_renderFieldErrors: function (fieldName, fieldErrors) {
			var form = this;
			_(fieldErrors).each(function (errorText) {
				$('<div class="field-error" data-fieldname="'+fieldName+'">'+errorText+'</div>').hide().insertAfter(form.$('input[name="'+fieldName+'"]')).show(100);
			});
		},

		_renderErrors: function () {
			var errors = this.tempModel.get('errors');
			this.$('.field-error').remove();
			if (errors && !_(errors).isEmpty()) {
				var form = this;
				var generalErrors = [];
				_(errors).each(function (fieldErrors, fieldName) {
					if (fieldName === '__all__') {
						generalErrors = generalErrors.concat(fieldErrors);
					} else if (form.$('input[name="'+fieldName+'"]').length > 0) {
						form._renderFieldErrors(fieldName, fieldErrors);
					} else {
						console.error("Got error for specific field but it wasn't actually in the form: ", fieldName, fieldErrors);
						generalErrors = generalErrors.concat(_(fieldErrors).map(function (e) { return fieldName+':'+e; }));
					}
				});

				if (generalErrors.length !== 0) {
					form._renderGeneralErrors(generalErrors);
				}
			} else {
				this.$('.general-errors').hide(100);
				this.$('.field-error').hide(100);
			}
		},

		render: function () {
			this.renderTemplate();
			this._renderErrors();
			// every form needs a submit input for HTML5 validation
			var submitElement = this.$('form input[type=submit]');
			if (submitElement.length === 0) {
				this.$('form').append('<input type="submit" style="display:none"/>');
			}
			return this;
		}
	});
}());
