/*global $*/
/*global _*/
/*global Toolkit*/
/*global _superApply*/
(function () {
	// Requires title and either content (plain html) or contentView (a renderable view)
	// triggers 'hide' event when it has been hidden
	// contentView can trigger 'cancel' event to hide the parent modal
	// dismissOnClickOutside: if false then clicking outside the modal will not close it. defaults to true.
	var ModalDialog = Toolkit.Views.ViewWithTemplate.extend({
		className: 'modal hide fade',
		template: 'component/modal/dialog',

		dismissOnClickOutside: true,

		templateAttributes: function () {
			return {
				title: this.options.title,
				cancellable: this.options.cancellable !== false
			};
		},
		show: function () {
			this.render();
			$('body').append(this.el);
			var modalOptions;
			if (!this.dismissOnClickOutside) {
				modalOptions = {backdrop: 'static'};
			}
			this.$el.modal(modalOptions);

			var self = this;

			// listen for bootstrap emitting 'hidden' event
			this.$el.on('hidden', function () {
				self._hidden = true;
				self.trigger('hide');
				self.close();
			});

			this._hidden = false;
		},
		close: function () {
			if (!this._hidden) {
				this.hide();
				return;
			}

			// actually cleanup the modal
			// should only be called when the modal is not visible
			ModalDialog.__super__.close.apply(this, arguments);
			this.options.contentView && this.options.contentView.close();
			this.options.onCancel && this.options.onCancel();
		},
		hide: function () {
			$(this.el).modal('hide');
		},
		initialize: function () {
			_superApply(ModalDialog, this, 'initialize', arguments);
			if (this.options.contentView) {
				this.listenTo(this.options.contentView, 'cancel', this.hide);
			}
			if ('dismissOnClickOutside' in this.options) {
				this.dismissOnClickOutside = this.options.dismissOnClickOutside;
			}
			this._hidden = true;
		},
		render: function () {
			_superApply(ModalDialog, this, 'render', arguments);

			if (this.options.contentView) {
				this.options.contentView.render();
				this.$('.modal-body').append(this.options.contentView.el);
				this.options.contentView.delegateEvents();
			}
			if (this.options.content) {
				this.$('.modal-body').html(this.options.content);
			}
			var footer = this.$('.modal-footer');
			if (this.options.actions) {
				_.each(this.options.actions, function (action) {
					if (action.type == "cancel") {
						footer.append('<button class="btn" data-dismiss="modal" aria-hidden="true">'+action.text+'</button>');
					} else {
						var button = $('<button class="btn">');
						button.html(action.text);
						if (action.type) {
							button.addClass('btn-'+action.type);
						}
						if (action.onClicked) {
							button.click(action.onClicked);
						}
						footer.append(button);
					}
				});
			} else {
				this.$('.modal-footer').remove();
			}
		}
	});

	Toolkit.Views.ModalDialog = ModalDialog;

	Toolkit.modal = {};
	
	/**
	 * Use this when you want window.alert, but don't want to stop JavaScript execution.
	 *
	 * @param options.title {string} title for alert box
	 * @param options.content {string} text for alert box
	 */
	Toolkit.modal.alert = function (options) {
		var modal = new Toolkit.Views.ModalDialog({
			title: options.title,
			content: '<p>'+options.content+'</p>',
			actions: [{
				text: "OK",
				onClicked: function () {
					modal.hide();
				}
			}]
		});
		modal.show();
		return modal;
	};
}());
