/*global Toolkit*/
/*global Backbone*/
(function () {
	Toolkit.Models.Platform = Backbone.Model.extend({
		idAttribute: 'version',
		getLocalConfigSchema: function () {
			var me = this;
			if (this.has('localConfigSchema')) {
				return;
			} else {
				var rpc = Toolkit.rpcSession.remoteCall("api.get", {
					method: "platform/"+this.id+"/local_config_schema",
					cookies: Toolkit.cookies
				});

				rpc.on('success', function (result) {
					var data = result.data['data'];
					me.set('localConfigSchema', data['schema']);
				});
			}
		}
	});

	Toolkit.Models.Platforms = Toolkit.Models.RpcModel.extend({
		method: 'api.get',
		args: {
			cookies: Toolkit.cookies,
			method: 'platform'
		},
		inject: {
			platforms: function (platforms) {
				return new Backbone.Collection(platforms, {
					model: Toolkit.Models.Platform
				});
			}
		},
		versionRegex: /v(\d+).(\d+)(?:.(\d+))?/,
		platformsLocalAppCanSwitchTo: function (localVersion) {
			var versionRegex = this.versionRegex;
			var majorVersion = localVersion.match(versionRegex);

			// if on e.g. default, just return the local version
			if (!majorVersion) {
				return [localVersion];
			}
			
			// add regex matches
			var needLocalVersion = true;

			var withMatches = this.get('platforms').chain().map(function (p) {
				// note if we see the local version
				if (p.get("version") === localVersion) {
					needLocalVersion = false;
				}

				var match = p.get("version").match(versionRegex);
				return {
					platform: p,
					match: match
				};
			});

			// filter those that didn't match
			var filtered = withMatches.filter(function (p) {
				var match = p.match;
				return match && match[1] === majorVersion[1] && match[2] === majorVersion[2];
			});

			// sort by minor platform, ascending with major platform first.
			var sorted = filtered.sort(function (p1, p2) {
				return (p1.match[3] || 0) - (p2.match[3] || 0);
			});

			// return just versions
			var versions = sorted.map(function (p) {
				return p.platform.get('version');
			}).value();

			if (needLocalVersion) {
				versions = versions.concat(localVersion);
			}

			return versions;
		}
	});
	Toolkit.platforms = new Toolkit.Models.Platforms();
	
	Toolkit.events.on('rpc', function () {
		Toolkit.registerPreReadyFunction(function (done) {
			Toolkit.platforms.load();
			Toolkit.platforms.once('loaded', done);
		});
	});
}());
