/*global _*/
/*global Toolkit*/
/*global CloseableView*/
/*global RpcCall*/
(function () {
	// requires a model, which should be JSON schema as an Object
	var SchemaForm = Toolkit.Views.ViewWithTemplate.extend({
		template: 'component/schemaform',

		events: {
			'keydown input': "keyDown",
			"click .submit": "submit",
			"click .cancel": "cancel"
		},

		createObjectView: function () {
			// remove the title, it's already in the modal dialog header.
			var schemaWithoutTitle = _.omit(this.model, 'title');
			var objectView = new Toolkit.Views.ConfigurationView({
				schema: schemaWithoutTitle,
				model: {},
				parent: this
			});
			objectView.render();
			return objectView;
		},

		keyDown: function (e) {
			if (e.which === 13) {
				this.submit(e);
			}
		},

		submit: function (e) {
			e.preventDefault();
			e.stopPropagation();
			this.$('button').focus();
			this.trigger("submit", {model: this.objectView.model, sensitive: this.objectView.options.sensitive});
		},

		cancel: function (e) {
			e.preventDefault();
			e.stopPropagation();
			this.trigger("cancel");
		},

		render: function () {
			this.renderTemplate();
			this.objectView = this.createObjectView();
			this.$('.object-view').html(this.objectView.el);
		},

		// this is called by the ConfigurationView whenever it saves
		// but we don't need to do anything
		save: function () {
		}
	});

	// requires a model, which should be JSON schema as an Object
	var ModalSchemaForm = CloseableView.extend({
		submitted: false,

		initialize: function () {
			ModalSchemaForm.__super__.initialize.apply(this, arguments);

			this.formView = new SchemaForm({
				model: this.model
			});
			this.autoClose(this.formView);

			this.modalView = new Toolkit.Views.ModalDialog({
				title: this.model.title,
				contentView: this.formView,
				dismissOnClickOutside: false
			});
			this.autoClose(this.modalView);

			this.listenTo(this.formView, 'submit', this.formViewSubmit);
			this.listenTo(this.modalView, 'hide', this.modalViewHide);
		},

		formViewSubmit: function (data) {
			this.submitted = true;
			this.trigger('submit', data);
			this.modalView.hide();
		},

		modalViewHide: function () {
			if (!this.submitted) {
				this.trigger('cancel');
			}
		},

		render: function () {
			this.$el.html('');
			this.modalView.render();
			this.$el.append(this.modalView.el);
		}
	});

	_.extend(RpcCall.prototype.defaults, {
		confirm: function (e, respond) {
			var actions = [{
				text: 'Confirm',
				type: 'primary',
				onClicked: function () {
					respond(true);
					modal.hide();
				}
			}];

			if (e.cancellable) {
				actions.push({
					text: 'Cancel',
					type: 'cancel'
				});
			}

			var modal = new Toolkit.Views.ModalDialog({
				title: e.title || 'Confirm Action',
				cancellable: e.cancellable,
				content: e.content || '<p><p>',
				onCancel: function () {
					// if not cancellable, send confirm if modal gets closed
					if (e.cancellable) {
						respond(false);
					} else {
						respond(true);
					}
				},
				actions: actions,
				dismissOnClickOutside: false
			});
			modal.show();

			// make current page view responsible for closing modal
			Toolkit.currentPageView.autoClose(modal);
		}
	});

	Toolkit.Views.ModalSchemaForm = ModalSchemaForm;
}());
