/* global Toolkit, Mustache, $ */
(function () {

	/**
	 * To use embeds on website pages do something like this:
	 *
	 * <iframe class="embed"
	 *         scrolling="no"
	 *         src="https://toolkit-local.com:{{request.session.toolkit_port}}/embed#embed/{{uuid}}/modules/{{name}}/{{version}}">
	 * </iframe>
	 */

	Toolkit.Views.EmbedModuleConfigView = Toolkit.Views.BaseConfigView.extend({
		template: 'embed/modules/config',

		render: function () {
			var local = this.model.get("app").get("local");
			if (local && local.get("problem")) {
				return this.$el.html(Mustache.to_html(Toolkit.Templates['apps/serious_problem'], {}));
			} else if (!Toolkit.Views.BaseConfigView.prototype.render.call(this)) {
				return this;
			}

			$('.spinner').removeClass('spinner-on');

			var name = this.model.get('name');
			var config = this.model.get('config').modules[name].config;
			var schema = this.model.get('schema');

			if (schema.properties) {
				// configuration view
				var configurationView = this.configurationView = new Toolkit.Views.ConfigurationView({
					model: config,
					schema: schema,
					pickerStart: local.get("path") + "/src",
					parent: this
				}).render();
				this.$(".configuration").append(configurationView.el);
			} else if (schema) {
				this.$(".no-config").show();
			}

			return this;
		},

		delegateSubViewEvents: function () {
			this.configurationView && this.configurationView.delegateEvents();
		},

		save: function () {
			Toolkit.Views.BaseConfigView.prototype.save.call(this);
		}
	});
})();
