/*global Toolkit*/
/*global Backbone*/
/*global _*/
/*jshint curly:false*/
Toolkit.Router = (function () {

	return Backbone.Router.extend({
		// before calling a route callback,
		// clean up event listeners set in routes using "listenTo"
		route: function (route, name, callback) {
			if (!callback) callback = this[name];
			return Toolkit.Router.__super__.route.call(this, route, name, function () {
				this.stopListening();
				// Bit hacky, prevent navigation away from auth/login while logged out.
				if (Toolkit.authStatus.get("status") == "loggedin" || route == "auth/login") {
					callback && callback.apply(this, arguments);
				} else if (Toolkit.authStatus.get("status") == "loggedout") {
					Toolkit.router.navigate("auth/login", true);
				}
			});
		},

		routes: {
			"": "projectList",
			"auth/login": "login",
			"auth/logout": "logout",

			"util/update": "updateRequired",

			"apps/:uuid": "appStatus",
			"apps/:uuid/forge": "appForge",
			"apps/:uuid/run/:target": "appRun",
			"apps/:uuid/package/:target": "appPackage",
			"apps/:uuid/serve/:target": "appServe",
			"apps/:uuid/config": "appConfig",
			"apps/:uuid/localConfig": "appLocalConfig",
			"apps/:uuid/plugins": "appPlugins",
			"apps/:uuid/browseModules": "browseModules",
			"apps/:uuid/browseModuleDetails/:module_name/:module_version": "browseModuleDetails",

			"apps/:uuid/reload": "reload",

			"projects/:id": "projectDetails",

			"plugins/:id": "pluginStatus",
			"plugins/:id/development": "pluginDevelopment",
			"plugins/:id/versions": "pluginVersions",

			"modules/:id": "moduleStatus",
			"modules/:id/versions": "moduleVersions",
			"modules/:id/upload": "moduleUpload",
			"modules/:id/manifest": "moduleManifest",
			"modules/:id/publish/:version": "modulePublish",
			"modules/:id/unpublish/:version": "moduleUnpublish",

			"apps/:uuid/projectModules": "projectModules",
			"apps/:uuid/publicModules": "publicModules",
			"modules/:id/projectModules": "moduleProjectModules",
			"modules/:id/publicModules": "modulePublicModules",
			"modules/:id/thisModule": "moduleThisModule",
		},

		browseModules: function (uuid) {
			var app = new Toolkit.Models.App({uuid: uuid}).load();
			var pageView = new Toolkit.Views.BrowseModulesView({
				model: app
			}).render();
			Toolkit.pageTransition(pageView);
		},

		browseModuleDetails: function (uuid, module_name, module_version) {
			var app = new Toolkit.Models.App({uuid: uuid}).load();
			var pageView = new Toolkit.Views.BrowseModulesView({
				model: app,
				name: module_name,
				version: module_version
			}).render();
			Toolkit.pageTransition(pageView);
		},

		pluginStatus: function (id) {
			if (!Toolkit.shouldNavigate()) return;

			var plugin = new Toolkit.Models.Plugin({uuid: id}).load();

			var pageView = new Toolkit.Views.PluginStatus({
				model: plugin
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/plugins/status');
		},

		pluginDevelopment: function (id) {
			if (!Toolkit.shouldNavigate()) return;

			var plugin = new Toolkit.Models.Plugin({uuid: id}).load();

			var pageView = new Toolkit.Views.PluginDevelopment({
				model: plugin
			}).render();
			Toolkit.pageTransition(pageView);

			Toolkit.trackPageview('/plugins/development');
		},

		pluginVersions: function (id) {
			if (!Toolkit.shouldNavigate()) return;

			var plugin = new Toolkit.Models.Plugin({uuid: id}).load();

			var pageView = new Toolkit.Views.PluginVersions({
				model: plugin
			}).render();
			Toolkit.pageTransition(pageView);

			Toolkit.trackPageview('/plugins/versions');
		},

		moduleStatus: function (id) {
			if (!Toolkit.shouldNavigate()) return;

			var module = new Toolkit.Models.Module({name: id}).load();

			var pageView = new Toolkit.Views.ModuleStatus({
				model: module
			}).render();
			Toolkit.pageTransition(pageView);

			Toolkit.trackPageview('/modules/status');
		},

		moduleVersions: function (id) {
			if (!Toolkit.shouldNavigate()) return;

			var module = new Toolkit.Models.Module({name: id}).load();

			var pageView = new Toolkit.Views.ModuleVersions({
				model: module
			}).render();
			Toolkit.pageTransition(pageView);

			Toolkit.trackPageview('/modules/versions');
		},

		moduleUpload: function (id) {
			if (!Toolkit.shouldNavigate()) return;

			var module = new Toolkit.Models.Module({name: id}).load();

			var pageView = new Toolkit.Views.ModuleUpload({
				model: module
			}).render();
			Toolkit.pageTransition(pageView);

			Toolkit.trackPageview('/modules/upload');
		},

		modulePublish: function (moduleName, version) {
			if (!Toolkit.shouldNavigate()) return;

			var module = new Toolkit.Models.Module({name: moduleName}).load();
			var model = new Backbone.Model({module: module});
			this.doWhenAvailable(module, 'versions', function (moduleVersions) {
				var moduleVersion = _.findWhere(moduleVersions, {version: version});
				model.set('moduleVersion', moduleVersion);
			});

			var pageView = new Toolkit.Views.ModulePublish({
				model: model
			}).render();
			Toolkit.pageTransition(pageView);

			Toolkit.trackPageview('/modules/publish');
		},

		moduleUnpublish: function (moduleName, version) {
			if (!Toolkit.shouldNavigate()) return;

			var module = new Toolkit.Models.Module({name: moduleName}).load();
			var model = new Backbone.Model({module: module});
			this.doWhenAvailable(module, 'versions', function (moduleVersions) {
				var moduleVersion = _.findWhere(moduleVersions, {version: version});
				model.set('moduleVersion', moduleVersion);
			});

			var pageView = new Toolkit.Views.ModuleUnpublish({
				model: model
			}).render();
			Toolkit.pageTransition(pageView);

			Toolkit.trackPageview('/modules/unpublish');
		},

		moduleManifest: function (id) {
			if (!Toolkit.shouldNavigate()) return;

			var module = new Toolkit.Models.Module({name: id}).load();

			var pageView = new Toolkit.Views.ModuleManifest({
				model: module
			}).render();
			Toolkit.pageTransition(pageView);

			Toolkit.trackPageview('/modules/manifest');
		},

		updateRequired: function () {
			if (!Toolkit.shouldNavigate()) return;
			var update = new Backbone.Model();
			var updateView = new Toolkit.Views.UpdateView({
				model: update
			});
			Toolkit.pageTransition(updateView.render());
			Toolkit.trackPageview('/util/update');

			var rpc = Toolkit.rpcSession.remoteCall('util.update');
			rpc.on('success', function () {
				update.set('status', 'complete');
			});
		},

		login: function () {
			if (!Toolkit.shouldNavigate()) return;

			var pageView = new Toolkit.Views.LoginView({
				model: Toolkit.authStatus
			}).render();
			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/login');
		},

		logout: function () {
			if (!Toolkit.shouldNavigate()) return;

			Toolkit.pageTransition(new Toolkit.Views.LogoutView().render());
			Toolkit.trackPageview('/logout');

			var rpc = Toolkit.rpcSession.remoteCall('auth.logout', {
				cookies: Toolkit.cookies
			});
			rpc.on('success', function () {
				console.log('Logged out.');
				// flush all persistent collections
				Toolkit.user.clear({silent: true});
				Toolkit.authStatus.set({status: 'loggedout'});
				Toolkit.router.navigate('auth/login', true);
			});
		},

		projectList: function () {
			if (!Toolkit.shouldNavigate()) return;

			Toolkit.user.load();

			var pageView = new Toolkit.Views.ProjectListView({
				model: Toolkit.user
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/projects/');
		},

		projectDetails: function (id) {
			if (!Toolkit.shouldNavigate()) return;

			var project = new Toolkit.Models.Project({id: id}).load();

			var pageView = new Toolkit.Views.ProjectDetailsView({
				model: project
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/projects/details');
		},

		_buildIfNecessary: function (app, cb) {
			app.doWhenAvailable(app, "local", function () {
				app.get("local").buildIfNecessary(cb);
			});
		},

		appForge: function (uuid) {
			if (!Toolkit.shouldNavigate()) return;

			var app = new Toolkit.Models.App({uuid: uuid}).load();
			this._buildIfNecessary(app, function () {
				var local = app.get("local");		

				// we need the schema to check forge live availability
				var schema_rpc = Toolkit.rpcSession.remoteCall('file.contents', {
					path: local.get('schemas').config
				});
				schema_rpc.on('success', function (e) {
					var schema = JSON.parse(e.data);
					local.set("configSchema", schema);
				});

				// we need the platform targets to fully determine plaform availability
				var platform_targets_rpc = Toolkit.rpcSession.remoteCall('app.platform_targets', {
					path: local.get('path')
				});
				platform_targets_rpc.on('success', function (e) {
					app.set("platform_targets", e.data);
				});
			});

			var pageView = new Toolkit.Views.AppForgeView({
				model: app
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/details');
		},

		appStatus: function (uuid) {
			if (!Toolkit.shouldNavigate()) return;

			var app = new Toolkit.Models.App({uuid: uuid}).load();
			this._buildIfNecessary(app);

			var pageView = new Toolkit.Views.AppStatusView({
				model: app
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/status');
		},

		_forgeCommand: function (uuid, command, target) {
			var app = new Toolkit.Models.App({uuid: uuid}).load();
			this._buildIfNecessary(app);

			var local = app.get("local");

			// create rpc
			var rpc = Toolkit.rpcSession.remoteCall('app.'+command, {
				cookies: Toolkit.cookies,
				path: local.get('path'),
				target: target
			});

			// wrap rpc in model for console view
			var consoleModel = new Toolkit.Models.ConsoleTask({
				rpc: rpc,
				messageWhenPreventingNavigation: "Currently performing '"+command+" "+target+"', leaving the page may cause it to be killed before it completes.",
				onForgeLogReceived: function () {
					Toolkit.trackPageview('/apps/'+command+'/'+target+'/success');
				}
			});

			// bundle up all info needed for view
			var consolePageModel = new Backbone.Model({
				name: local.get('config').name,
				uuid: app.get('uuid'),
				target: target,
				command: command,
				consoleModel: consoleModel,
				app: app
			});

			var pageView = new Toolkit.Views.AppConsoleView({
				model: consolePageModel
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/'+command+'/'+target);
		},

		appRun: function (uuid, target) {
			if (!Toolkit.shouldNavigate()) return;
			if (!Toolkit.authStatus.get('validated_email')) return;
			this._forgeCommand(uuid, 'run', target);
		},

		appPackage: function (uuid, target) {
			if (!Toolkit.shouldNavigate()) return;
			if (!Toolkit.authStatus.get('validated_email')) return;
			this._forgeCommand(uuid, 'package', target);
		},

		appServe: function (uuid, target) {
			if (!Toolkit.shouldNavigate()) return;
			if (!Toolkit.authStatus.get('validated_email')) return;
			this._forgeCommand(uuid, 'serve', target);
		},

		appConfig: function (uuid) {
			if (!Toolkit.shouldNavigate()) return;

			var app = new Toolkit.Models.App({uuid: uuid}).load();

			var local = app.get("local");
			var model = new Backbone.Model({
				uuid: uuid,
				app: app,
				path: local.get('files').config
			});

			this._buildIfNecessary(app, function () {
				var rpc = Toolkit.rpcSession.remoteCall('file.contents', {
					path: local.get('schemas').config
				});
				rpc.on('success', function (e) {
					var schema = JSON.parse(e.data);
					model.set({schema: schema, config: local.get('config')});
				});
			});


			// get premium modules for v1.4
			Toolkit.premium.load();
			this.doWhenAvailable(Toolkit.premium, "modules", function() {
				this.doWhenAvailable(app, "project", function (project) {
					this.doWhenAvailable(project, "features", function () {
						var premium_modules = Toolkit.premium.get("modules").map(function (module) {
							return module.get("name");
						});
						premium_modules = _.filter(premium_modules, function (module) {
							return !_.contains(project.get("features").modules, module);
						});
						model.set({
							premium_modules: premium_modules
						});
					});
				});
			});

			var pageView = new Toolkit.Views.AppConfigView({
				model: model
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/config');
		},

		appLocalConfig: function (uuid) {
			if (!Toolkit.shouldNavigate()) return;
			var app = new Toolkit.Models.App({uuid: uuid}).load();
			this._buildIfNecessary(app);

			var local = app.get("local");
			var platform = Toolkit.platforms.get('platforms').get(local.get('config') && local.get('config').platform_version);

			if (platform) {
				platform.getLocalConfigSchema();
			}

			var model = new Backbone.Model({
				uuid: uuid,
				app: app,
				config: local.get('localConfig'),
				path: local.get('files').localConfig,
				platform: platform
			});

			var pageView = new Toolkit.Views.AppLocalConfigView({
				model: model
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/localConfig');
		},

		appPlugins: function (uuid) {
			if (!Toolkit.shouldNavigate()) return;

			var app = new Toolkit.Models.App({uuid: uuid}).load();
			this._buildIfNecessary(app);

			var local = app.get("local");

			var pageView = new Toolkit.Views.AppPluginsView({
				model: app,
				// pass in copy as config is mutated by the view!
				config: JSON.parse(JSON.stringify(local.get('config')))
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/plugins');
		},

		reload: function (uuid) {
			if (!Toolkit.shouldNavigate()) return;
			var app = new Toolkit.Models.App({uuid: uuid}).load();
			this._buildIfNecessary(app);
			try {
				app.loadBuildEvents(true);
			} catch (e) {

			}

			var pageView = new Toolkit.Views.ReloadView({
				model: app
			}).render();
			Toolkit.trackPageview('/apps/reload');

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/reload');
		},

		reloadRestore: function (uuid, hash) {
			if (!Toolkit.shouldNavigate()) return;
			var app = new Toolkit.Models.App({uuid: uuid}).load();
			var buildEvent = new Toolkit.Models.BuildEvent({config_hash: hash}).load();

			var pageView = new Toolkit.Views.ReloadRestoreConfigView({
				model: app,
				buildevent: buildEvent
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/reload/restore');
		},

		reloadCreateStream: function (uuid) {
			if (!Toolkit.shouldNavigate()) return;
			var app = new Toolkit.Models.App({uuid: uuid}).load();
			app.loadBuildEvents(true);

			var pageView = new Toolkit.Views.ReloadCreateStreamView({
				model: app
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/reload/create');
		},

		reloadPush: function (uuid, hash, stream) {
			if (!Toolkit.shouldNavigate()) return;
			var app = new Toolkit.Models.App({uuid: uuid}).load();
			var buildEvent = new Toolkit.Models.BuildEvent({config_hash: hash}).load();

			var model = new Backbone.Model({
				app: app,
				buildevent: buildEvent,
				stream: stream
			});

			var pageView = new Toolkit.Views.ReloadPushView({
				model: model
			}).render();

			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/reload/push');
		},

		_modules: function (PageView, uuid) {
			if (!Toolkit.shouldNavigate()) return;

			var app = new Toolkit.Models.App({uuid: uuid}).load();

			var model = new Backbone.Model({
				app: app,
				config_modules: app.get("config_modules"),
				local: app.get("local")
			});

			this.doWhenAvailable(app, 'project', function (project) {
				model.set("project", project);
			});

			this._buildIfNecessary(app, function () {});

			var pageView = new PageView({
				model: model
			});
			pageView.render();

			Toolkit.pageTransition(pageView);
		},

		projectModules: function (uuid) {
			this._modules(Toolkit.Views.ProjectModulesView, uuid);
		},

		publicModules: function (uuid) {
			this._modules(Toolkit.Views.ModulesView, uuid);
		},

		_inspectorModules: function (PageView, id) {
			if (!Toolkit.shouldNavigate()) return;

			var module = new Toolkit.Models.Module({name: id}).load();

			var model = new Backbone.Model({
				module: module
			});
			this.doWhenAvailable(module, 'project', function (project) {
				model.set("project", project);
			});
			this.doWhenAvailable(module, 'local', function (local) {
				model.set("local", local);
			});
			this.doWhenAvailable(module, 'config_modules', function (config_modules) {
				model.set("config_modules", config_modules);
			});

			var pageView = new PageView({
				model: model
			});
			pageView.render();

			Toolkit.pageTransition(pageView);
		},

		moduleProjectModules: function (id) {
			this._inspectorModules(Toolkit.Views.InspectorProjectModulesView, id);
		},

		modulePublicModules: function (id) {
			this._inspectorModules(Toolkit.Views.InspectorModulesView, id);
		},

		moduleThisModule: function (id) {
			this._inspectorModules(Toolkit.Views.InspectorThisModuleView, id);
		}
	});
}());
