/*global Toolkit*/
/*global $*/
/*global Backbone*/
/*jshint curly:false*/
Toolkit.Router = (function () {

	return Backbone.Router.extend({
		// before calling a route callback,
		// clean up event listeners set in routes using "listenTo"
		route: function (route, name, callback) {
			if (!callback) callback = this[name];
			return Toolkit.Router.__super__.route.call(this, route, name, function () {
				this.stopListening();
				// Bit hacky, prevent navigation away from auth/login while logged out.
				if (Toolkit.authStatus.get("status") == "loggedin") {
					callback && callback.apply(this, arguments);
				} else if (Toolkit.authStatus.get("status") == "loggedout") {
					Toolkit.router.navigate("embed/loggedout", true);
				}
			});
		},

		routes: {
			"": "blankEmbed",
			"embed/loggedout": "loggedOut",
			"embed/:uuid/modules/:name/v:version": "embedModuleConfig"
		},

		blankEmbed: function () {

		},

		loggedOut: function () {
			alert("Embedded toolkit while logged out!");
		},

		embedModuleConfig: function (uuid, name, version) {
			if (!Toolkit.shouldNavigate()) return;
			var app = new Toolkit.Models.App({uuid: uuid}).load();
			this._buildIfNecessary(app);
			var model = new Backbone.Model({
				name: name,
				app: app
			});
			this.doWhenAvailable(app, 'local', function (local) {
				this.doWhenAvailable(local, 'config', function () {
					model.set({
						config: local.get('config'),
						path: local.get('files').config
					});
				});
				this.doWhenAvailable(app, "project", function (project) {
					var rpc = Toolkit.rpcSession.remoteCall("api.get", {
						method: "module_version?" + $.param({
							project_id: project.id,
							module_name: name,
							module_version: version
						}),
						cookies: Toolkit.cookies
					});
					rpc.on('success', function (result) {
						var schema = result.data.data.schema;
						model.set({
							schema: schema
						});
					});
				});
			});
			var pageView = new Toolkit.Views.EmbedModuleConfigView({
				model: model
			}).render();
			Toolkit.pageTransition(pageView);
			Toolkit.trackPageview('/apps/moduleConfig');
		},

		_buildIfNecessary: function (app, cb) {
			app.doWhenAvailable(app, "local", function () {
				app.get("local").buildIfNecessary(cb);
			});
		}

	});
}());
