/*global Toolkit, _, makeSet*/
(function () {
	Toolkit.preprocess = {
		// Take a target name and return a dict with the name and a human readable title
		target: function (name) {
			var titles = {
				'android': 'Android',
				'chrome': 'Chrome',
				'ios': 'iOS',
				'firefox': 'Firefox',
				'ie': 'Internet Explorer',
				'safari': 'Safari',
				'web': 'Web',
				'osx': 'OS X',
				'crosswalk': 'Crosswalk'
			};
			return {
				title: titles[name] || name,
				name: name
			};
		},

		module: function (module) {
			_.each(module['versions'], this.moduleVersion);
		},

		moduleVersion: function (moduleVersion) {
			// allow mustache to see which approval state
			moduleVersion['approval'] = makeSet(moduleVersion['approval']);

			moduleVersion['clashesGroupedByModuleName'] = _(moduleVersion['clashes'])
				.chain()
				.groupBy(function (clash) {
					return clash.module_version_to__module__name;
				})
				.map(function (v, k) {
					return {name: k, versions: v};
				})
				.value();
		},

		// Take a Module and return a dict usable by a mustache template
		// which wants to show a manifest/dependency editor UI
		moduleManifest: function (module) {
			var local = module.get("local");
			var manifest = local && local.get("manifest");

			if (!manifest) {
				return {};
			}

			var project = module.get("project");
			var dependencies = project && project.get("dependencies");

			var manifest_dependencies = [];
			_.each(manifest.dependencies, function (data, dependency) {
				var versions = _.pluck(_.where(dependencies, {module: dependency}), 'version');
				var dep = {
					module: dependency,
					minimum_versions: [{value: "any", selected: !data.minimum_version}],
					maximum_versions: [{value: "any", selected: !data.maximum_version}]
				};

				_.each(versions, function (version) {
					dep.minimum_versions.push({
						value: version,
						selected: data.minimum_version == version
					});
					dep.maximum_versions.push({
						value: version,
						selected: data.maximum_version == version
					});
				});

				manifest_dependencies.push(dep);
			});

			var available_platforms = [];
			available_platforms = [{
				platform: 'any',
				selected_min: !manifest.min_platform_version,
				selected_max: !manifest.max_platform_version
			}];
			Toolkit.platforms.get('platforms').each(function (platform) {
				if (platform.get('module_support')) {
					available_platforms.push({
						platform: platform.get('version'),
						selected_min: manifest.min_platform_version == platform.get('version'),
						selected_max: manifest.max_platform_version == platform.get('version')
					});
				}
			});

			var available_dependency_names = _.unique(_.pluck(dependencies, 'module'));

			return {
				available_dependency_names: available_dependency_names,
				manifest_dependencies: manifest_dependencies,
				available_platforms: available_platforms
			};
		}
	};
})();