/*global Toolkit*/
/*global _*/
/*global Mustache*/
/*global $*/
/*global CloseableView*/
/*global _superApply*/
/*global Markdown*/

(function () {
	var Views = Toolkit.Views;
	var Templates = Toolkit.Templates;

	var ViewWithTemplate = Views.ViewWithTemplate = CloseableView.extend({
		modelAttributes: function () {
			if (this.model) {
				return JSON.parse(JSON.stringify(this.model)) || {};
			} else {
				return {};
			}
		},

		templateAttributes: function () {
			return this.modelAttributes();
		},

		renderFeatureStyle: function (project) {
			var css = '.feature { display: none }\n';

			if (!project) {
				return this;
			}

			project.get('features') && _.each(project.get('features')['toolkit'], function (feature) {
				css += '.feature_'+feature+'_enabled { display: block }\n';
				css += '.feature_'+feature+'_enabled_hide { display: none }\n';
			});

			this.$el.prepend($('<style>').text(css));
			return this;
		},

		renderTemplate: function (templateOverride) {
			var templateName = templateOverride || this.template;
			var template = Templates[templateName];
			if (!template) {
				throw "No such template: " + templateName;
			}

			var viewAttributes = this.templateAttributes();
			var attributes = _.extend(
				_.clone(ViewWithTemplate.globalTemplateAttributes),
				viewAttributes
			);

			var html = Mustache.to_html(template, attributes, Toolkit.Templates);

			this.$el.html(html);
			
			try {
				this._lastRenderHash = JSON.stringify(this.templateAttributes());
			} catch (e) {}

			return this;
		},

		render: function () {
			_superApply(ViewWithTemplate, this, 'render', arguments);
			this.renderTemplate();
			return this;
		},

		renderIfNecessary: function () {
			if (!this._lastRenderHash || this._lastRenderHash != JSON.stringify(this.templateAttributes())) {
				return this.render();
			} else {
				return this;
			}
		}
	}, {
		globalTemplateAttributes: {},

		addGlobalTemplateAttribute: function (attr, value) {
			this.globalTemplateAttributes[attr] = value;
		}
	});

	ViewWithTemplate.addGlobalTemplateAttribute('websiteUrl', Toolkit.websiteUrl);
	ViewWithTemplate.addGlobalTemplateAttribute('markdown', function () {
		return function (text, render) {
			return (new Markdown.Converter()).makeHtml(render(text));
		};
	});

	Views.LoginView = ViewWithTemplate.extend({
		breadcrumbs: function () {
			return [];
		},
		className: 'login',
		template: 'auth/login',
		templateAttributes: function () {
			return {
				server: Toolkit.server,
				port: window.location.port
			};
		}
	});

	Views.LogoutView = ViewWithTemplate.extend({
		breadcrumbs: function () {
			return [];
		},
		className: 'login',
		template: 'auth/logout'
	});

	var GlobalNavigationView = Views.GlobalNavigationView = ViewWithTemplate.extend({
		className: 'masthead thin',
		template: 'util/globalnav',

		initialize: function () {
			_superApply(GlobalNavigationView, this, 'initialize', arguments);
			Toolkit.authStatus.bind('change', this.render, this);
		},

		render: function () {
			var authStatus = Toolkit.authStatus.get('status');
			this.renderTemplate();
			if (authStatus == "loggedin") {
				this.$('.loggedinas').text('Logged in as '+Toolkit.authStatus.get('user'));
				this.$('.logout').show();
			} else {
				this.$('.loggedinas').text('');
				this.$('.logout').hide();
			}
			return this;
		}
	});

	Views.ErrorView = ViewWithTemplate.extend({
		template: 'component/error'
	});

	Views.IframeSessionView = ViewWithTemplate.extend({
		template: "util/iframe/session"
	});

	Views.FailedToLoadWebsiteView = ViewWithTemplate.extend({
		template: "util/failed_to_load_website"
	});

	// take two subviews and put them into a sidebar/content layout
	var SideBarLayoutView = Toolkit.Views.SideBarLayoutView = Toolkit.Views.ViewWithTemplate.extend({
		template: "component/sidebarlayout",

		initialize: function () {
			SideBarLayoutView.__super__.initialize.call(this);
			this.autoClose(this.options.sidebar);
			this.autoClose(this.options.content);
		},

		render: function () {
			this.renderTemplate();
			this.$('aside').html(this.options.sidebar.render().el);
			this.$('section').html(this.options.content.render().el);
			return this;
		}
	});

	Toolkit.Views.SideBarView = Toolkit.Views.ViewWithTemplate.extend({
		template: "component/sidebar"
	});
}());
