﻿import markdown
from markdown.util import etree
import unicodedata
import re

# Borrowed from headerid
def slugify(value, separator):
	""" Slugify a string, to make it URL friendly. """
	value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore')
	value = re.sub('[^\w\s-]', '', value.decode('ascii')).strip().lower()
	return re.sub('[%s\s]+' % separator, separator, value)

class ApiProcessor(markdown.blockprocessors.BlockProcessor):
	""" Process Tables. """

	def test(self, parent, block):
		if not block.strip().startswith("!method"):
			return False

		lines = block.split('\n')

		for line in lines:
			if not line.strip().startswith("!"):
				return False

		return True

	def run(self, parent, blocks):
		""" Parse a table block and build table. """
		api = {
			"params": []
		}

		block = blocks.pop(0).split('\n')
		for line in block:
			line = line.strip()[1:]
			linetype = line[:line.find(":")]
			linecontent = line[len(linetype)+1:].strip()

			if linetype == "param":
				api["params"].append(linecontent)
			else:
				api[linetype] = linecontent

		kw = {'class': 'api'}
		blockquote = etree.SubElement(parent, 'blockquote', **kw)
		slug = slugify(api['method'], '-')
		kw = {'id': slug}
		h4 = etree.SubElement(blockquote, 'h4', **kw)
		h4.text = api['method']
		kw = {'href': '#%s' % slug}
		a = etree.SubElement(h4, 'a', **kw)
		a.text = u'¶'
		if "platforms" in api:
			small_p = etree.SubElement(blockquote, 'p')
			small = etree.SubElement(small_p, 'small')
			small.text = api['platforms']
		if "description" in api:
			p = etree.SubElement(parent, 'p')
			p.text = api['description']
		if len(api['params']) > 0:
			kw = {'class': 'dl-normal'}
			dl = etree.SubElement(parent, 'dl', **kw)
			dt = etree.SubElement(dl, 'dt')
			dt.text = 'Parameters:'
			dd = etree.SubElement(dl, 'dd')
			kw = {'class': 'dl-horizontal'}
			subdl = etree.SubElement(dd, 'dl', **kw)
			for param in api['params']:
				param_name = param[:param.find(" ")]
				param_desc = param[len(param_name)+1:]
				param_dt = etree.SubElement(subdl, 'dt')
				param_dt.text = param_name
				param_dd = etree.SubElement(subdl, 'dd')
				param_dd.text = param_desc

class ApiExtension(markdown.Extension):
	""" Add tables to Markdown. """

	def extendMarkdown(self, md, md_globals):
		""" Add an instance of TableProcessor to BlockParser. """
		md.parser.blockprocessors.add('api', 
									  ApiProcessor(md.parser),
									  '<hashheader')
