import markdown
from markdown.inlinepatterns import Pattern
from markdown.util import etree

class EmphasisPattern(Pattern):
	def handleMatch(self, m):
		note_type = m.group(2).lower()
		text = m.group(2)

		if note_type == 'warning':
			icon = 'warning-sign'
			style = 'note-warning'
		elif note_type == 'important':
			icon = 'exclamation-sign'
			style = 'note-important'
		else:
			icon = 'info-sign'
			style = 'note-note'

		kw = {'class': style}
		el = etree.Element('span', **kw)
		kw = {'class': 'icon-%s icon-large' % icon}
		etree.SubElement(el, 'i', **kw)
		strong = etree.SubElement(el, 'strong')
		strong.text = " %s:" % text
		return el

MYPATTERN = r'::([^:]+)::'

class NoteExtension(markdown.extensions.Extension):
	""" Add smart_emphasis extension to Markdown class."""

	def extendMarkdown(self, md, md_globals):
		""" Modify inline patterns. """
		md.inlinePatterns.add('note', EmphasisPattern(MYPATTERN), '>emphasis2')