import os
import logging
import json
import hashlib
import shutil
import tempfile
from StringIO import StringIO

from trigger import forge_tool
from trigger.api import expose, run_in_process
from trigger.api.app import build
from trigger.lib import make_hash
from forge import async
from forge import build_config
from forge import main as forge_main
from forge.lib import cd

LOG = logging.getLogger(__name__)

@expose
def restore_config(path, config):
    '''Replace an app at "paths"s config with "config"'''
    config = json.loads(config)
    if 'uuid' in config:
        config.pop('uuid')
    with async.current_call().io_lock:
        with open(os.path.join(path, "src", "config.json"), 'w') as config_file:
            json.dump(config, config_file, indent=4, sort_keys=True)
    return config

@run_in_process
def push(cookies, path, stream_name):
    # Override cookies for any new remote objects
    from forge import remote
    remote.override_cookies = cookies

    build(cookies, path, 'reload')
    with cd(path):
        forge_main.handle_primary_options(['reload', 'push', stream_name])
        forge_main.handle_secondary_options('reload', ['push', stream_name])
        forge_main.reload(['push', stream_name])
@run_in_process
def pushcdn(cookies, path, stream_name, manifest_url):
    # Override cookies for any new remote objects
    from forge import remote
    remote.override_cookies = cookies

    build(cookies, path, 'reload')
    with cd(path):
        forge_main.handle_primary_options(['reload', 'pushcdn', stream_name, manifest_url])
        forge_main.handle_secondary_options('reload', ['pushcdn', stream_name, manifest_url])
        forge_main.reload(['pushcdn', stream_name, manifest_url])

    return "ok"


@expose
def generate_manifest(cookies, url, path):
    if not url.endswith('/'):
        url = url + '/'

    # We need the forged content, not the original files!
    build(cookies, path, 'reload')

    manifest = {}
    src_path = os.path.join(path, "development", "reload", "src")

    output_path = tempfile.mkdtemp('reload_manifest')

    for root, dirs, files in os.walk(src_path):
        for index, dir_ in enumerate(dirs):
            if dir_ in ('.hg', '.git',):
                del dirs[index]

        for file_ in files:
            full_path = os.path.join(root, file_)
            rel_path = os.path.relpath(full_path, src_path)
            with async.current_call().io_lock:
                with open(full_path) as in_file:
                    hash_ = hashlib.sha1()
                    while True:
                        buf = in_file.read(0x100000)
                        if not buf:
                            break
                        hash_.update(buf)
            hash_value = hash_.hexdigest()

            shutil.copyfile(full_path, os.path.join(output_path, hash_value))
            manifest[rel_path] = url + hash_value
            print "copied", full_path, "to", os.path.join(output_path, hash_value)

    manifest_str = json.dumps(manifest, indent=4)
    manifest_hash = hashlib.sha1(manifest_str).hexdigest()
    manifest_path = os.path.join(output_path, manifest_hash)

    with async.current_call().io_lock:
        with open(manifest_path, "w") as manifest_file:
            manifest_file.write(manifest_str)
            print "created", manifest_path

    return { 'output_path': output_path,
                     'manifest_path': manifest_path,
                     'manifest_url': url + manifest_hash }
