"""Contains the entry point for the Trigger Toolkit."""


import sys

# relaunch from library directory so that any updated toolkit files don't
# conflict with the original distribution
if sys.platform.startswith("darwin") and sys.argv[0].find("Library/Trigger Toolkit") == -1:
    print "Relaunching toolkit from library directory"
    path = sys.modules["os.path"]
    subprocess = sys.modules["subprocess"]
    TriggerToolkit = path.join(path.expanduser("~"), "Library", "Trigger Toolkit", "TriggerToolkit")
    subprocess.Popen(TriggerToolkit)
    sys.exit(0)


import logging
from os import path

import requests

import workspace
from forge import updater
from trigger import lib

LOG = logging.getLogger(__name__)

def main(launch_browser=True, port=None):
    """Be wary when changing the signature for this method, it is expected by the TriggerToolkit binary to accept
    at least the following positional args:

    :param launch_browser: If true, try to open the preferred web browser after starting the toolkit server.
    :param port: If this is not None, use the given port for the toolkit server (only really used for testing purposes).
    """
    lib.setup_log_to_file('startup.log', backup=True)
    try:
        # Check for port file
        if '--testing' not in sys.argv:
            port_file_path = workspace.path('.trigger_port')
            if path.exists(port_file_path):
                with open(port_file_path, 'r') as port_file:
                    previous_port = int(port_file.read())

                try:
                    r = requests.get('https://%s:%d/' % ('toolkit-local.com', previous_port), timeout=0.5)
                    if r.status_code == 200:
                        LOG.info('Existing toolkit detected at port: %d' % previous_port)
                        url = 'https://%s:%d/' % ('toolkit-local.com', previous_port)
                        if launch_browser:
                            lib.open_url(url)
                        return
                except:
                    pass

        updater.update_and_restart_if_necessary()

        # delay import to prevent locking binary dependencies
        from trigger import webserver
        webserver.run_server(launch_browser)
    except Exception:
        LOG.error("Uncaught exception in main", exc_info=True)
        sys.exit(1)
